﻿# Why Refactor
A magic number is a numeric value that is encountered in the source but has no obvious meaning. This “anti-pattern” makes it harder to understand the program and refactor the code.

Yet more difficulties arise when you need to change this magic number. Find and replace won’t work for this: the same number may be used for different purposes in different places, meaning that you will have to verify every line of code that uses this number.

# Benefits
* The symbolic constant can serve as live documentation of the meaning of its value.
* It is much easier to change the value of a constant than to search for this number throughout the entire codebase, without the risk of accidentally changing the same number used elsewhere for a different purpose.
* Reduce duplicate use of a number or string in the code. This is especially important when the value is complicated and long (such as 3.14159 or 0xCAFEBABE).

# Drawbacks
Sometimes seemingly useless temps are used to cache the result of an expensive operation that is reused several times. So before using this refactoring technique, make sure that simplicity will not come at the cost of performance.

# Good to Know
Not all numbers are magical. If the purpose of a number is obvious, there is no need to replace it. A classic example is:

`for (i = 0; i < сount; i++) ... ;`

# How to Refactor
1. Declare a constant and assign the value of the magic number to it.
2. Find all mentions of the magic number.
3. For each of the numbers that you find, double-check that the magic number in this particular case corresponds to the purpose of the constant. If yes, replace the number with your constant. This is an important step, since the same number can mean absolutely different things (and replaced with different constants, as the case may be).

# Sources
* https://sourcemaking.com/refactoring/replace-magic-number-with-symbolic-constant
* Martin Fowler, Kent Beck: Refactoring, Improving the Design of Existing Code 

# Rider Support
* Shortcut: Ctrl + Alt + F
* Refactor -> Introduce Field... 
* See: https://www.jetbrains.com/help/rider/Refactorings__Introduce_Field.html